# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

import six
# Imports ###########################################################
from lxml import etree
from xblock.core import XBlock
from xblock.fields import Scope, String
from xblock.fragment import Fragment
from xblockutils.studio_editable import StudioEditableXBlockMixin

from problem_builder.mixins import XBlockWithTranslationServiceMixin


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


@XBlock.needs("i18n")
class MentoringMessageBlock(XBlock, StudioEditableXBlockMixin, XBlockWithTranslationServiceMixin):
    """
    A message which can be conditionally displayed at the mentoring block level,
    for example upon completion of the block
    """
    MESSAGE_TYPES = {
        "completed": {
            "display_name": _(u"Completed"),
            "studio_label": _(u'Message (Complete)'),
            "long_display_name": _(u"Message shown when complete"),
            "default": _(u"Great job!"),
            "description": _(
                u"This message will be shown when the student achieves a perfect score. "
            ),
        },
        "incomplete": {
            "display_name": _(u"Incomplete"),
            "studio_label": _(u'Message (Incomplete)'),
            "long_display_name": _(u"Message shown when incomplete"),
            "default": _(u"Not quite! You can try again, though."),
            "description": _(
                u"This message will be shown when the student gets at least one question wrong, "
                "but is allowed to try again. "
            ),
        },
        "max_attempts_reached": {
            "display_name": _(u"Reached max. # of attempts"),
            "studio_label": _(u'Message (Max # Attempts)'),
            "long_display_name": _(u"Message shown when student reaches max. # of attempts"),
            "default": _(u"Sorry, you have used up all of your allowed submissions."),
            "description": _(
                u"This message will be shown when the student has used up "
                "all of their allowed attempts without achieving a perfect score. "
            ),
        },
        "on-assessment-review-question": {
            "display_name": _(u"Study tips if this question was wrong"),
            "long_display_name": _(u"Study tips shown if question was answered incorrectly"),
            "default": _(
                u"Review ____."
            ),
            "description": _(
                u"This message will be shown when the student is reviewing "
                "their answers to the assessment, if the student got this specific question "
                "wrong and is allowed to try again."
            ),
        },
    }

    has_author_view = True

    content = String(
        display_name=_("Message"),
        help=_("Message to display upon completion"),
        scope=Scope.content,
        default="",
        multiline_editor="html",
        resettable_editor=False,
    )
    type = String(
        help=_("Type of message"),
        scope=Scope.content,
        default="completed",
        values=(
            {"value": "completed", "display_name": MESSAGE_TYPES["completed"]["display_name"]},
            {"value": "incomplete", "display_name": MESSAGE_TYPES["incomplete"]["display_name"]},
            {"value": "max_attempts_reached", "display_name": MESSAGE_TYPES["max_attempts_reached"]["display_name"]},
            {
                "value": "on-assessment-review-question",
                "display_name": MESSAGE_TYPES["on-assessment-review-question"]["display_name"]
            },
        ),
    )
    editable_fields = ("content", )

    def mentoring_view(self, context=None):
        """ Render this message for use by a mentoring block. """
        html = u'<div class="submission-message {msg_type}">{content}</div>'.format(
            msg_type=self.type,
            content=self.content
        )
        return Fragment(html)

    def student_view(self, context=None):
        """ Normal view of this XBlock, identical to mentoring_view """
        return self.mentoring_view(context)

    def author_view(self, context=None):
        fragment = self.mentoring_view(context)
        fragment.content += u'<div class="submission-message-help"><p>{}</p></div>'.format(self.help_text)
        return fragment

    @property
    def display_name_with_default(self):
        try:
            return self._(self.MESSAGE_TYPES[self.type]["long_display_name"])
        except KeyError:
            return u"INVALID MESSAGE"

    @property
    def help_text(self):
        try:
            return self._(self.MESSAGE_TYPES[self.type]["description"])
        except KeyError:
            return u"This message is not a valid message type!"

    @classmethod
    def get_template(cls, template_id):
        """
        Used to interact with Studio's create_xblock method to instantiate pre-defined templates.
        """
        return {'data': {
            'type': template_id,
            'content': cls.MESSAGE_TYPES[template_id]["default"],
        }}

    @classmethod
    def parse_xml(cls, node, runtime, keys, id_generator):
        """
        Construct this XBlock from the given XML node.
        """
        block = runtime.construct_xblock_from_class(cls, keys)
        block.content = six.text_type(node.text or u"")
        if 'type' in node.attrib:  # 'type' is optional - default is 'completed'
            block.type = node.attrib['type']
        for child in node:
            block.content += etree.tostring(child, encoding='unicode')

        return block


class CompletedMentoringMessageShim(object):
    CATEGORY = 'pb-message'
    STUDIO_LABEL = _("Message (Complete)")


class IncompleteMentoringMessageShim(object):
    CATEGORY = 'pb-message'
    STUDIO_LABEL = _("Message (Incomplete)")


def get_message_label(type):
    return MentoringMessageBlock.MESSAGE_TYPES[type]['studio_label']



import json
import unittest
from datetime import datetime

import pytz
from mock import MagicMock, Mock
from xblock.core import XBlock
from xblock.field_data import DictFieldData
from xblock.fields import Boolean, DateTime, Integer, Scope, String

from problem_builder.mixins import StudentViewUserStateMixin


class NoUserStateFieldsMixin(object):
    scope_settings = String(name="Field1", scope=Scope.settings)
    scope_content = String(name="Field1", scope=Scope.content)
    user_state_summary = String(name="Not in the output", scope=Scope.user_state_summary)


class UserStateFieldsMixin(object):
    answer_1 = String(name="state1", scope=Scope.user_state)
    answer_2 = Boolean(name="state2", scope=Scope.user_state)

    preference_1 = String(name="pref1", scope=Scope.preferences)
    preference_2 = Integer(name="pref2", scope=Scope.preferences)

    user_info_1 = String(name="info1", scope=Scope.user_info)
    user_info_2 = DateTime(name="info2", scope=Scope.user_info)


class ChildrenMixin(object):
    # overriding children for ease of testing
    _children = []
    has_children = True

    @property
    def children(self):
        return self._children

    @children.setter
    def children(self, value):
        self._children = value


class XBlockWithNoUserState(XBlock, NoUserStateFieldsMixin, StudentViewUserStateMixin):
    pass


class XBlockNoChildrenWithUserState(XBlock, NoUserStateFieldsMixin, UserStateFieldsMixin, StudentViewUserStateMixin):
    pass


class XBlockChildrenNoUserState(XBlock, NoUserStateFieldsMixin, ChildrenMixin, StudentViewUserStateMixin):
    has_children = True


class XBlockChildrenUserState(
    XBlock, NoUserStateFieldsMixin, UserStateFieldsMixin, ChildrenMixin, StudentViewUserStateMixin
):
    has_children = True


class TestStudentViewUserStateMixin(unittest.TestCase):
    def setUp(self):
        self._runtime = MagicMock()

    def _build_block(self, block_type, fields):
        return block_type(self._runtime, DictFieldData(fields), Mock())

    def _set_children(self, block, children):
        block.children = children.keys()
        self._runtime.get_block.side_effect = children.get

    def _merge_dicts(self, dict1, dict2):
        result = dict1.copy()
        result.update(dict2)
        return result

    def test_no_user_state_returns_empty(self):
        block = self._build_block(XBlockWithNoUserState, {"scope_settings": "qwe", "scope_content": "ASD"})

        self.assertEqual(block.build_user_state_data(), {})

    def test_no_child_blocks_with_user_state(self):
        user_fields = {
            "answer_1": "AAAA",
            "answer_2": False,
            "preference_1": "Yes",
            "preference_2": 12,
            "user_info_1": "John",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        other_fields = {"setting": "setting", "content": "content", "user_state_summary": "Something"}
        block_fields = self._merge_dicts(user_fields, other_fields)
        block = self._build_block(XBlockNoChildrenWithUserState, block_fields)
        block.USER_STATE_FIELDS = list(user_fields.keys())

        self.assertEqual(block.build_user_state_data(), user_fields)

    def test_only_shows_whitelisted_fields(self):
        user_fields = {
            "answer_1": "AAAA",
            "answer_2": False,
        }
        other_fields = {"setting": "setting", "content": "content", "user_state_summary": "Something"}
        block_fields = self._merge_dicts(user_fields, other_fields)
        block = self._build_block(XBlockNoChildrenWithUserState, block_fields)
        block.USER_STATE_FIELDS = ['answer_1']

        self.assertEqual(block.build_user_state_data(), {
            'answer_1': 'AAAA'
        })

    def test_transform(self):
        """
        Transform should only affect fields listed in USER_STATE_FIELDS,
        and should only return functions that accept one parameter.
        """
        block_fields = {
            "answer_1": "AAAA",
            "answer_2": False,
        }
        block = self._build_block(XBlockNoChildrenWithUserState, block_fields)
        block.USER_STATE_FIELDS = ['answer_1']

        def transforms():
            return {
                'answer_1': lambda value: value.replace('A', 'B'),
                'answer_2': lambda value: True
            }

        block.transforms = transforms
        self.assertEqual(block.build_user_state_data(), {
            'answer_1': 'BBBB'
        })

        def bad_transforms():
            return {
                'answer_1': lambda: 'Fail'
            }

        block.transforms = bad_transforms
        with self.assertRaises(TypeError):
            block.build_user_state_data()

    def test_children_empty_no_user_state(self):
        block = self._build_block(XBlockChildrenNoUserState, {"scope_settings": "qwe", "scope_content": "ASD"})
        self.assertEqual(block.children, [])  # precondition

        self.assertEqual(block.build_user_state_data(), {"components": {}})

    def test_children_no_user_state(self):
        block = self._build_block(XBlockChildrenNoUserState, {"scope_settings": "qwe", "scope_content": "ASD"})
        no_user_state1 = self._build_block(XBlockWithNoUserState, {"scope_settings": "qwe", "scope_content": "ASD"})
        no_user_state2 = self._build_block(XBlockWithNoUserState, {"scope_settings": "ZXC", "scope_content": "VBN"})
        nested = {"child1": no_user_state1, "child2": no_user_state2}
        self._set_children(block, nested)

        # preconditions
        self.assertEqual(block.children, nested.keys())
        self.assertEqual(self._runtime.get_block("child1"), no_user_state1)
        self.assertEqual(self._runtime.get_block("child2"), no_user_state2)

        student_user_state = block.build_user_state_data()

        expected = {"components": {"child1": {}, "child2": {}}}
        self.assertEqual(student_user_state, expected)

    def test_children_with_user_state(self):
        block = self._build_block(XBlockChildrenNoUserState, {"scope_settings": "qwe", "scope_content": "ASD"})

        other_fields = {"setting": "setting", "content": "content", "user_state_summary": "Something"}
        user_fields1 = {
            "answer_1": "AAAA",
            "answer_2": False,
            "preference_1": "Yes",
            "preference_2": 12,
            "user_info_1": "John",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        block.USER_STATE_FIELDS = list(user_fields1.keys())

        user_fields2 = {
            "answer_1": "BBBB",
            "answer_2": True,
            "preference_1": "No",
            "preference_2": 7,
            "user_info_1": "jane",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        user_state1 = self._build_block(XBlockNoChildrenWithUserState, self._merge_dicts(user_fields1, other_fields))
        user_state2 = self._build_block(XBlockNoChildrenWithUserState, self._merge_dicts(user_fields2, other_fields))
        nested = {"child1": user_state1, "child2": user_state2}
        self._set_children(block, nested)

        # preconditions
        self.assertEqual(block.children, nested.keys())
        self.assertEqual(self._runtime.get_block("child1"), user_state1)
        self.assertEqual(self._runtime.get_block("child2"), user_state2)

        student_user_state = block.build_user_state_data()

        expected = {"components": {"child1": {}, "child2": {}}}
        self.assertEqual(student_user_state, expected)

    def test_user_state_at_parent_and_children(self):
        other_fields = {"setting": "setting", "content": "content", "user_state_summary": "Something"}
        user_fields = {
            "answer_1": "OOOO",
            "answer_2": True,
            "preference_1": "IDN",
            "preference_2": 42,
            "user_info_1": "Douglas",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        block = self._build_block(XBlockChildrenUserState, self._merge_dicts(user_fields, other_fields))
        block.USER_STATE_FIELDS = list(user_fields.keys())

        nested_user_fields = {
            "answer_1": "AAAA",
            "answer_2": False,
            "preference_1": "Yes",
            "preference_2": 12,
            "user_info_1": "John",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        user_state = self._build_block(
            XBlockNoChildrenWithUserState, self._merge_dicts(nested_user_fields, other_fields)
        )
        nested = {"child1": user_state}
        self._set_children(block, nested)

        # preconditions
        self.assertEqual(block.children, nested.keys())
        self.assertEqual(self._runtime.get_block("child1"), user_state)

        student_user_state = block.build_user_state_data()

        expected = user_fields.copy()
        expected["components"] = {"child1": {}}
        self.assertEqual(student_user_state, expected)

    def test_user_state_handler(self):
        other_fields = {"setting": "setting", "content": "content", "user_state_summary": "Something"}
        user_fields = {
            "answer_1": "OOOO",
            "answer_2": True,
            "preference_1": "IDN",
            "preference_2": 42,
            "user_info_1": "Douglas",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        block = self._build_block(XBlockChildrenUserState, self._merge_dicts(user_fields, other_fields))
        block.USER_STATE_FIELDS = list(user_fields.keys())

        nested_user_fields = {
            "answer_1": "AAAA",
            "answer_2": False,
            "preference_1": "Yes",
            "preference_2": 12,
            "user_info_1": "John",
            "user_info_2": datetime(2017, 1, 2, 3, 4, 5, tzinfo=pytz.UTC)
        }
        user_state = self._build_block(
            XBlockNoChildrenWithUserState, self._merge_dicts(nested_user_fields, other_fields)
        )
        nested = {"child1": user_state}
        self._set_children(block, nested)

        # preconditions
        self.assertEqual(block.children, nested.keys())
        self.assertEqual(self._runtime.get_block("child1"), user_state)

        block.USER_STATE_FIELDS = ['answer_1', 'answer_2', 'preference_1', 'preference_2', 'user_info_1', 'user_info_2']
        student_user_state_response = block.student_view_user_state()
        student_user_state = json.loads(student_user_state_response.body.decode('utf-8'))

        expected = user_fields.copy()
        expected["user_info_2"] = expected["user_info_2"].isoformat()
        expected["components"] = {"child1": {}}
        self.assertEqual(student_user_state, expected)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import os
import sys
from setuptools import setup, find_packages
from setuptools.command.install import install


# Constants #########################################################

VERSION = '3.4.22'

# Functions #########################################################


def package_data(pkg, root_list):
    """Generic function to find package_data for `pkg` under `root`."""
    data = []
    for root in root_list:
        for dirname, _, files in os.walk(os.path.join(pkg, root)):
            for fname in files:
                data.append(os.path.relpath(os.path.join(dirname, fname), pkg))

    return {pkg: data}


class VerifyTagCommand(install):
    """Custom command to verify that the git tag matches the current version."""
    description = 'verify that the git tag matches the current version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != 'v{}'.format(VERSION):
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


# Main ##############################################################

BLOCKS = [
    'problem-builder = problem_builder.mentoring:MentoringBlock',
    'step-builder = problem_builder.mentoring:MentoringWithExplicitStepsBlock',
    'sb-step = problem_builder.step:MentoringStepBlock',
    'sb-review-step = problem_builder.step_review:ReviewStepBlock',
    'sb-conditional-message = problem_builder.step_review:ConditionalMessageBlock',
    'sb-review-score = problem_builder.step_review:ScoreSummaryBlock',
    'sb-review-per-question-feedback = problem_builder.step_review:PerQuestionFeedbackBlock',

    'sb-plot = problem_builder.plot:PlotBlock',
    'sb-plot-overlay = problem_builder.plot:PlotOverlayBlock',

    'pb-table = problem_builder.table:MentoringTableBlock',
    'pb-column = problem_builder.table:MentoringTableColumn',
    'pb-answer = problem_builder.answer:AnswerBlock',
    'pb-answer-recap = problem_builder.answer:AnswerRecapBlock',
    'pb-mcq = problem_builder.mcq:MCQBlock',
    'pb-swipe = problem_builder.swipe:SwipeBlock',
    'pb-rating = problem_builder.mcq:RatingBlock',
    'pb-mrq = problem_builder.mrq:MRQBlock',
    'pb-slider = problem_builder.slider:SliderBlock',
    'pb-completion = problem_builder.completion:CompletionBlock',
    'pb-message = problem_builder.message:MentoringMessageBlock',
    'pb-tip = problem_builder.tip:TipBlock',
    'pb-choice = problem_builder.choice:ChoiceBlock',

    'pb-dashboard = problem_builder.dashboard:DashboardBlock',
    'pb-data-export = problem_builder.instructor_tool:InstructorToolBlock',  # Deprecated; use 'pb-instructor-tool' now
    'pb-instructor-tool = problem_builder.instructor_tool:InstructorToolBlock',
]

setup(
    name='xblock-problem-builder',
    version=VERSION,
    description='XBlock - Problem Builder',
    packages=find_packages(),
    install_requires=[
        'XBlock>=1.2',
        'xblock-utils',
    ],
    entry_points={
        'xblock.v1': BLOCKS,
    },
    package_data=package_data("problem_builder", ["templates", "public", "migrations", "locale"]),
    cmdclass={
        'verify_tag': VerifyTagCommand,
    },
)



import unittest
from random import random

import ddt
from mock import MagicMock, Mock, PropertyMock, patch
from xblock.field_data import DictFieldData

from problem_builder.answer import AnswerRecapBlock
from problem_builder.mcq import MCQBlock
from problem_builder.mentoring import (MentoringBlock, MentoringMessageBlock,
                                       _default_options_config)
from problem_builder.mrq import MRQBlock

from .utils import BlockWithChildrenTestMixin


@ddt.ddt
class TestMRQBlock(BlockWithChildrenTestMixin, unittest.TestCase):
    def test_student_view_data(self):
        """
        Ensure that all expected fields are always returned.
        """
        block = MRQBlock(Mock(), DictFieldData({}), Mock())

        self.assertEqual(
            sorted(block.student_view_data().keys()),
            sorted([
                'hide_results', 'tips', 'block_id', 'display_name',
                'weight', 'title', 'question', 'message', 'type', 'id', 'choices'
            ])
        )


@ddt.ddt
class TestAnswerRecapBlock(BlockWithChildrenTestMixin, unittest.TestCase):
    def test_student_view_data(self):
        """
        Ensure that all expected fields are always returned.
        """
        block = AnswerRecapBlock(Mock(), DictFieldData({}), Mock())

        self.assertEqual(
            sorted(block.student_view_data().keys()),
            sorted([
                'block_id', 'display_name', 'type', 'id', 'name', 'description'
            ])
        )


@ddt.ddt
class TestMentoringBlock(BlockWithChildrenTestMixin, unittest.TestCase):
    def test_sends_progress_event_when_rendered_student_view_with_display_submit_false(self):
        block = MentoringBlock(MagicMock(), DictFieldData({
            'display_submit': False
        }), Mock())

        with patch.object(block, 'runtime') as patched_runtime:
            patched_runtime.publish = Mock()

            block.student_view(context={})

            patched_runtime.publish.assert_called_once_with(block, 'progress', {})

    def test_does_not_send_progress_event_when_rendered_student_view_with_display_submit_true(self):
        block = MentoringBlock(MagicMock(), DictFieldData({
            'display_submit': True
        }), Mock())

        with patch.object(block, 'runtime') as patched_runtime:
            patched_runtime.publish = Mock()

            block.student_view(context={})

            self.assertFalse(patched_runtime.publish.called)

    @ddt.data(True, False)
    def test_get_content_titles(self, has_title_set):
        """
        Test that we don't send a title to the LMS for the sequential's tooltips when no title
        is set
        """
        if has_title_set:
            data = {'display_name': 'Custom Title'}
            expected = ['Custom Title']
        else:
            data = {}
            expected = []
        block = MentoringBlock(MagicMock(), DictFieldData(data), Mock())
        self.assertEqual(block.get_content_titles(), expected)

    def test_does_not_crash_when_get_child_is_broken(self):
        block = MentoringBlock(MagicMock(), DictFieldData({
            'children': ['invalid_id'],
        }), Mock())

        with patch.object(block, 'runtime') as patched_runtime:
            patched_runtime.publish = Mock()
            patched_runtime.service().ugettext = lambda str: str
            patched_runtime.get_block = lambda block_id: None
            patched_runtime.load_block_type = lambda block_id: Mock

            fragment = block.student_view(context={})

            self.assertIn('Unable to load child component', fragment.content)

    @ddt.data(
        (True, True, True),
        (True, False, False),
        (False, False, True),
        (False, False, True),
    )
    @ddt.unpack
    def test_correctly_decides_to_show_or_hide_feedback_message(
            self, pb_hide_feedback_if_attempts_remain, max_attempts_reached, expected_show_message
    ):
        block = MentoringBlock(Mock(), DictFieldData({
            'student_results': ['must', 'be', 'non-empty'],
        }), Mock())
        block.get_option = Mock(return_value=pb_hide_feedback_if_attempts_remain)
        with patch(
                'problem_builder.mentoring.MentoringBlock.max_attempts_reached', new_callable=PropertyMock
        ) as patched_max_attempts_reached:
            patched_max_attempts_reached.return_value = max_attempts_reached
            _, _, show_message = block._get_standard_results()
            self.assertEqual(show_message, expected_show_message)

    def test_allowed_nested_blocks(self):
        block = MentoringBlock(Mock(), DictFieldData({}), Mock())
        self.assert_allowed_nested_blocks(block, message_blocks=[
                'pb-message',  # Message type: "completed"
                'pb-message',  # Message type: "incomplete"
                'pb-message',  # Message type: "max_attempts_reached"
            ]
        )

    def test_student_view_data(self):
        def get_mock_components():
            child_a = Mock(spec=['student_view_data'])
            child_a.block_id = 'child_a'
            child_a.category = 'text'
            child_a.student_view_data.return_value = 'child_a_json'
            child_b = Mock(spec=[])
            child_b.block_id = 'child_b'
            child_b.category = 'html'
            return [child_a, child_b]
        shared_data = {
            'max_attempts': 3,
            'extended_feedback': True,
            'feedback_label': 'Feedback label',
        }
        children = get_mock_components()
        children_by_id = {child.block_id: child for child in children}
        display_name = "I'm problem builder"
        block_data = {'display_name': display_name, 'children': children}
        block_data.update(shared_data)
        block = MentoringBlock(Mock(usage_id=1), DictFieldData(block_data), Mock(usage_id=1))
        block.runtime = Mock(
            get_block=lambda block: children_by_id[block.block_id],
            load_block_type=lambda block: Mock,
            id_reader=Mock(get_definition_id=lambda block: block, get_block_type=lambda block: block),
        )
        expected = {
            'block_id': '1',
            'display_name': display_name,
            'components': ['child_a_json'],
            'messages': {
                'completed': None,
                'incomplete': None,
                'max_attempts_reached': None,
            }
        }
        expected.update(shared_data)
        self.assertEqual(block.student_view_data(), expected)


@ddt.ddt
class TestMentoringBlockOptions(unittest.TestCase):
    def setUp(self):
        self.service_mock = Mock()
        self.runtime_mock = Mock()
        self.runtime_mock.service = Mock(return_value=self.service_mock)
        self.block = MentoringBlock(self.runtime_mock, DictFieldData({}), Mock())

    def test_get_options_returns_default_if_settings_service_is_not_available(self):
        self.runtime_mock.service = Mock(return_value=None)
        self.assertEqual(self.block.get_options(), _default_options_config)

    def test_get_options_returns_default_if_xblock_settings_not_customized(self):
        self.block.get_xblock_settings = Mock(return_value=None)
        self.assertEqual(self.block.get_options(), _default_options_config)
        self.block.get_xblock_settings.assert_called_once_with(default={})

    @ddt.data(
        {}, {'mass': 123}, {'spin': {}}, {'parity': "1"}
    )
    def test_get_options_returns_default_if_options_not_customized(self, xblock_settings):
        self.block.get_xblock_settings = Mock(return_value=xblock_settings)
        self.assertEqual(self.block.get_options(), _default_options_config)
        self.block.get_xblock_settings.assert_called_once_with(default={})

    @ddt.data(
        {MentoringBlock.options_key: 123},
        {MentoringBlock.options_key: [1, 2, 3]},
        {MentoringBlock.options_key: {'pb_mcq_hide_previous_answer': False}},
     )
    def test_get_options_correctly_returns_customized_options(self, xblock_settings):
        self.block.get_xblock_settings = Mock(return_value=xblock_settings)
        self.assertEqual(self.block.get_options(), xblock_settings[MentoringBlock.options_key])
        self.block.get_xblock_settings.assert_called_once_with(default={})

    def test_get_option(self):
        random_key, random_value = random(), random()
        with patch.object(self.block, 'get_options') as patched_get_options:
            # Happy path: Customizations contain expected key
            patched_get_options.return_value = {random_key: random_value}
            option = self.block.get_option(random_key)
            patched_get_options.assert_called_once_with()
            self.assertEqual(option, random_value)
        with patch.object(self.block, 'get_options') as patched_get_options:
            # Sad path: Customizations do not contain expected key
            patched_get_options.return_value = {}
            option = self.block.get_option(random_key)
            patched_get_options.assert_called_once_with()
            self.assertEqual(option, None)

    def test_student_view_calls_get_option(self):
        random_key, random_value = random(), random()
        self.block.get_xblock_settings = Mock(return_value={})
        with patch.object(self.block, 'get_option') as patched_get_option:
            self.runtime_mock.service.return_value = {
                random_key: random_value,
            }
            self.block.student_view({})
            patched_get_option.assert_any_call('pb_mcq_hide_previous_answer')
            patched_get_option.assert_any_call('pb_hide_feedback_if_attempts_remain')

    def test_get_standard_results_calls_get_option(self):
        with patch.object(self.block, 'get_option') as patched_get_option:
            self.block._get_standard_results()
            patched_get_option.assert_called_with('pb_hide_feedback_if_attempts_remain')


class TestMentoringBlockJumpToIds(unittest.TestCase):
    def setUp(self):
        self.service_mock = Mock()
        self.runtime_mock = Mock()
        self.runtime_mock.service = Mock(return_value=self.service_mock)
        self.block = MentoringBlock(self.runtime_mock, DictFieldData({}), Mock())
        self.block.children = ['dummy_id']
        self.message_block = MentoringMessageBlock(
            self.runtime_mock, DictFieldData({'type': 'bogus', 'content': 'test'}), Mock()
        )
        self.block.runtime.replace_jump_to_id_urls = lambda x: x.replace('test', 'replaced-url')

    def test_get_message_content(self):
        with patch('problem_builder.mixins.child_isinstance') as mock_child_isinstance:
            mock_child_isinstance.return_value = True
            self.runtime_mock.get_block = Mock()
            self.runtime_mock.get_block.return_value = self.message_block
            self.assertEqual(self.block.get_message_content('bogus'), 'replaced-url')

    def test_get_tip_content(self):
        self.mcq_block = MCQBlock(self.runtime_mock, DictFieldData({'name': 'test_mcq'}), Mock())
        self.mcq_block.get_review_tip = Mock()
        self.mcq_block.get_review_tip.return_value = self.message_block.content
        self.assertEqual(self.block.review_tips, [])

    def test_get_tip_content_no_tips(self):
        self.mcq_block = MCQBlock(self.runtime_mock, DictFieldData({'name': 'test_mcq'}), Mock())
        self.mcq_block.get_review_tip = Mock()
        # If there are no review tips, get_review_tip will return None;
        # simulate this situation here:
        self.mcq_block.get_review_tip.return_value = None
        self.block.step_ids = []
        self.block.steps = [self.mcq_block]
        self.block.student_results = {'test_mcq': {'status': 'incorrect'}}
        try:
            review_tips = self.block.review_tips
        except TypeError:
            self.fail('Trying to replace jump_to_id URLs in non-existent review tips.')
        else:
            self.assertEqual(review_tips, [])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import logging

import six
from xblock.core import XBlock
from xblock.fields import UNSET, JSONField, Scope, String
from xblock.fragment import Fragment
from xblockutils.resources import ResourceLoader
from xblockutils.studio_editable import StudioEditableXBlockMixin

from .mixins import (QuestionMixin, StudentViewUserStateMixin,
                     XBlockWithTranslationServiceMixin)
from .sub_api import SubmittingXBlockMixin, sub_api

# Globals ###########################################################

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text


# Classes ###########################################################


class NullableBoolean(JSONField):
    """
    A field class for representing a boolean which may also be ``null``,
    indicating that a value has not been assigned.

    The value, as loaded or enforced, can be either a Python bool, ``None``,
    a string, or any value that will then be converted to a bool in the
    ``from_json`` method.

    Examples:
1
    ::

        True -> True
        'true' -> True
        'TRUE' -> True
        'any other string' -> False
        [] -> False
        ['123'] -> True
        None - > None

    """
    # We're OK redefining built-in `help`
    def __init__(self, help=None, default=UNSET, scope=Scope.content, display_name=None,
                 **kwargs):  # pylint: disable=redefined-builtin
        super(NullableBoolean, self).__init__(help, default, scope, display_name,
                                              values=({'display_name': "True", "value": True},
                                                      {'display_name': "False", "value": False},
                                                      {'display_name': "Null", "value": None}),
                                              **kwargs)

    def from_json(self, value):
        if value is None:
            return None
        if isinstance(value, str):
            value = value.decode('ascii', errors='replace')
        if isinstance(value, six.text_type):
            return value.lower() == 'true'
        else:
            return bool(value)

    enforce_type = from_json


@XBlock.needs('i18n')
class CompletionBlock(
    SubmittingXBlockMixin, QuestionMixin, StudioEditableXBlockMixin, XBlockWithTranslationServiceMixin,
    StudentViewUserStateMixin, XBlock
):
    """
    An XBlock used by students to indicate that they completed a given task.
    The student's answer is always considered "correct".
    """
    CATEGORY = 'pb-completion'
    STUDIO_LABEL = _(u'Completion')
    USER_STATE_FIELDS = ['student_value']

    answerable = True

    question = String(
        display_name=_('Question'),
        help=_('Mentions a specific activity and asks the student whether they completed it.'),
        scope=Scope.content,
        default=_(
            'Please indicate whether you attended the In Person Workshop session by (un-)checking the option below.'
        ),
    )

    answer = String(
        display_name=_('Answer'),
        help=_(
            'Represents the answer that the student can (un-)check '
            'to indicate whether they completed the activity that the question mentions.'
        ),
        scope=Scope.content,
        default=_('Yes, I attended the session.'),
    )

    student_value = NullableBoolean(
        help=_("Records student's answer."),
        scope=Scope.user_state,
        default=None,
    )

    editable_fields = ('display_name', 'show_title', 'question', 'answer')

    def mentoring_view(self, context):
        """
        Main view of this block.
        """
        context = context.copy() if context else {}
        context['question'] = self.question
        context['answer'] = self.answer
        context['checked'] = self.student_value if self.student_value is not None else False
        context['title'] = self.display_name_with_default
        context['hide_header'] = context.get('hide_header', False) or not self.show_title

        html = loader.render_template('templates/html/completion.html', context)

        fragment = Fragment(html)
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/completion.js'))
        fragment.initialize_js('CompletionBlock')
        return fragment

    student_view = mentoring_view
    preview_view = mentoring_view

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course XBlock API.
        """
        return {
            'id': self.name,
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
            'question': self.question,
            'answer': self.answer,
            'title': self.display_name_with_default,
            'hide_header': not self.show_title,
        }

    def get_last_result(self):
        """ Return the current/last result in the required format """
        if self.student_value is None:
            return {}
        return {
            'submission': self.student_value,
            'status': 'correct',
            'tips': [],
            'weight': self.weight,
            'score': 1,
        }

    def get_results(self):
        """ Alias for get_last_result() """
        return self.get_last_result()

    def submit(self, value):
        """
        Persist answer submitted by student.
        """
        log.debug(u'Received Completion submission: "%s"', value)
        self.student_value = value
        if sub_api:
            # Also send to the submissions API:
            sub_api.create_submission(self.student_item_key, value)
        result = self.get_last_result()
        log.debug(u'Completion submission result: %s', result)
        return result



import copy
import unittest

from mock import MagicMock, Mock
from xblock.field_data import DictFieldData

from problem_builder.mentoring import MentoringBlock


class TestFieldMigration(unittest.TestCase):
    """
    Test mentoring fields data migration
    """

    def test_partial_completion_status_migration(self):
        """
        Changed `completed` to `status` in `self.student_results` to accomodate partial responses
        """
        # Instantiate a mentoring block with the old format
        student_results = [
            [u'goal',
                {u'completed': True,
                 u'score': 1,
                 u'student_input': u'test',
                 u'weight': 1}],
            [u'mcq_1_1',
                {u'completed': False,
                 u'score': 0,
                 u'submission': u'maybenot',
                 u'weight': 1}],
        ]
        mentoring = MentoringBlock(MagicMock(), DictFieldData({'student_results': student_results}), Mock())
        self.assertEqual(copy.deepcopy(student_results), mentoring.student_results)

        migrated_student_results = copy.deepcopy(student_results)
        migrated_student_results[0][1]['status'] = 'correct'
        migrated_student_results[1][1]['status'] = 'incorrect'
        del migrated_student_results[0][1]['completed']
        del migrated_student_results[1][1]['completed']
        mentoring.migrate_fields()
        self.assertEqual(migrated_student_results, mentoring.student_results)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

from .base_test import MentoringBaseTest

# Classes ###########################################################


class AnswerBlockTest(MentoringBaseTest):
    def test_answer_edit(self):
        """
        Answers of same name should share value accross blocks
        """

        # Answer should initially be blank on all instances with the same answer name
        mentoring = self.go_to_page('Answer Edit 2')
        answer1_bis = mentoring.find_element_by_css_selector('textarea')
        answer1_readonly = mentoring.find_element_by_css_selector('blockquote.answer.read_only')
        self.assertEqual(answer1_bis.get_attribute('value'), '')
        self.assertEqual(answer1_readonly.text, 'No answer yet.')

        # Another answer with the same name
        mentoring = self.go_to_page('Answer Edit 1')

        # Check <html> child
        p = mentoring.find_element_by_css_selector('p')
        self.assertEqual(p.text, 'This should be displayed in the answer_edit scenario')

        # Initial unsubmitted text
        answer1 = mentoring.find_element_by_css_selector('textarea')
        self.assertEqual(answer1.text, '')

        # Submit is disabled for empty answer
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')
        self.assertFalse(submit.is_enabled())

        # Filling in the answer enables the submit button
        answer1.send_keys('This is the answer')
        self.assertTrue(submit.is_enabled())
        submit.click()
        self.wait_until_disabled(submit)

        self.assertEqual(answer1.get_attribute('value'), 'This is the answer')

        # Modifying the answer re-enables submission
        answer1.send_keys('. It has a second statement.')
        self.assertTrue(submit.is_enabled())

        # Submitting a new answer overwrites the previous one
        submit.click()
        self.wait_until_disabled(submit)
        self.assertEqual(answer1.get_attribute('value'), 'This is the answer. It has a second statement.')

        # Answer content should show on a different instance with the same name
        mentoring = self.go_to_page('Answer Edit 2')
        answer1_bis = mentoring.find_element_by_css_selector('textarea')
        answer1_readonly = mentoring.find_element_by_css_selector('blockquote.answer.read_only')
        self.assertEqual(answer1_bis.get_attribute('value'), 'This is the answer. It has a second statement.')
        self.assertEqual(answer1_readonly.text, 'This is the answer. It has a second statement.')

    def test_answer_blank_read_only(self):
        """
        Read-only answers should not prevent completion when blank
        """
        # Check initial state
        mentoring = self.go_to_page('Answer Blank Read Only')
        answer = mentoring.find_element_by_css_selector('blockquote.answer.read_only')
        self.assertEqual(answer.text, 'No answer yet.')

        # Submit should allow to complete
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')
        self.assertTrue(submit.is_enabled())
        submit.click()

        # Submit is disabled after submission
        self.wait_until_disabled(submit)



"""
Unit tests for AnswerMixin.
"""
import json
import unittest
from collections import namedtuple
from datetime import datetime

from django.utils.crypto import get_random_string

from problem_builder.answer import AnswerMixin
from problem_builder.models import Answer


class TestAnswerMixin(unittest.TestCase):
    """ Unit tests for AnswerMixin. """

    FakeRuntime = namedtuple('FakeRuntime', ['course_id', 'anonymous_student_id'])

    def setUp(self):
        self.course_id = 'course-v1:edX+DemoX+Demo_Course'
        self.anonymous_student_id = '12345678987654321'

    def make_answer_mixin(self, name=None, course_id=None, student_id=None):
        if name is None:
            name = get_random_string()
        if course_id is None:
            course_id = self.course_id
        if student_id is None:
            student_id = self.anonymous_student_id
        answer_mixin = AnswerMixin()
        answer_mixin.name = name
        answer_mixin.runtime = self.FakeRuntime(course_id, student_id)
        answer_mixin.fields = {}
        answer_mixin.has_children = False
        return answer_mixin

    def test_creates_model_instance(self):
        name = 'test-model-creation'
        answer_mixin = self.make_answer_mixin(name=name)
        model = answer_mixin.get_model_object()
        self.assertEqual(model.name, name)
        self.assertEqual(model.student_id, self.anonymous_student_id)
        self.assertEqual(model.course_key, self.course_id)
        self.assertEqual(Answer.objects.get(pk=model.pk), model)

    def test_finds_instance_by_course_key(self):
        name = 'test-course-key'
        existing_model = Answer(
            name=name,
            student_id=self.anonymous_student_id,
            course_key=self.course_id,
        )
        existing_model.save()
        answer_mixin = self.make_answer_mixin(name=name)
        model = answer_mixin.get_model_object()
        self.assertEqual(model, existing_model)

    def test_works_with_long_course_keys(self):
        course_id = 'course-v1:VeryLongOrganizationName+VeryLongCourseNumber+VeryLongCourseRun'
        self.assertTrue(len(course_id) > 50)  # precondition check
        answer_mixin = self.make_answer_mixin(course_id=course_id)
        model = answer_mixin.get_model_object()
        self.assertEqual(model.course_key, course_id)

    def test_build_user_state_data(self):
        name = 'test-course-key-2'
        existing_model = Answer(
            name=name,
            student_id=self.anonymous_student_id,
            course_key=self.course_id,
            student_input="Test",
            created_on=datetime(2017, 1, 2, 3, 4, 5),
            modified_on=datetime(2017, 7, 8, 9, 10, 11),
        )
        existing_model.save()
        answer_mixin = self.make_answer_mixin(name=name)
        student_view_user_state = answer_mixin.build_user_state_data()

        expected_user_state_data = {
            "student_input": existing_model.student_input,
        }
        self.assertEqual(student_view_user_state, expected_user_state_data)

    def test_student_view_user_state(self):
        name = 'test-course-key-3'
        existing_model = Answer(
            name=name,
            student_id=self.anonymous_student_id,
            course_key=self.course_id,
            student_input="Test",
            created_on=datetime(2017, 1, 2, 3, 4, 5),
            modified_on=datetime(2017, 7, 8, 9, 10, 11),
        )
        existing_model.save()
        answer_mixin = self.make_answer_mixin(name=name)
        student_view_user_state = answer_mixin.student_view_user_state()

        parsed_student_state = json.loads(student_view_user_state.body.decode('utf-8'))

        expected_user_state_data = {
            "student_input": existing_model.student_input,
        }
        self.assertEqual(parsed_student_state, expected_user_state_data)



# -*- coding: utf-8 -*-
#


# Make '_' a no-op so we can scrape strings
def _(text):
    return text


def ngettext_fallback(text_singular, text_plural, number):
    """ Dummy `ngettext` replacement to make string extraction tools scrape strings marked for translation """
    if number == 1:
        return text_singular
    else:
        return text_plural


class DummyTranslationService(object):
    """
    Dummy drop-in replacement for i18n XBlock service
    """
    _catalog = {}
    gettext = _
    ngettext = ngettext_fallback


class I18NService(object):
    """
    Add i18n_service attribute to XBlocks
    """
    @property
    def i18n_service(self):
        """ Obtains translation service """
        return self.runtime.service(self, "i18n") or DummyTranslationService()



import unittest

import ddt
from mock import Mock
from xblock.field_data import DictFieldData

from problem_builder.swipe import SwipeBlock


@ddt.ddt
class TestSwipeBlock(unittest.TestCase):
    def test_student_view_data(self):
        """
        Ensure that all expected fields are always returned with
        appropriate values.
        """
        runtime = Mock(replace_urls=lambda url: '"{}"'.format(url))
        block = SwipeBlock(runtime, DictFieldData({"display_name": "Test"}), Mock())

        self.assertEqual(block.student_view_data()['correct'], False)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import logging

import six
from xblock.fields import Boolean, List, Scope, String
from xblock.validation import ValidationMessage
from xblockutils.resources import ResourceLoader

from problem_builder.mixins import (ExpandStaticURLMixin,
                                    StudentViewUserStateMixin)

from .questionnaire import QuestionnaireAbstractBlock
from .sub_api import SubmittingXBlockMixin, sub_api

# Globals ###########################################################

log = logging.getLogger(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


class MRQBlock(SubmittingXBlockMixin, StudentViewUserStateMixin, QuestionnaireAbstractBlock, ExpandStaticURLMixin):
    """
    An XBlock used to ask multiple-response questions
    """
    CATEGORY = 'pb-mrq'
    STUDIO_LABEL = _(u"Multiple Response Question")
    USER_STATE_FIELDS = ['student_choices', ]

    student_choices = List(
        # Last submissions by the student
        default=[],
        scope=Scope.user_state
    )
    required_choices = List(
        display_name=_("Required Choices"),
        help=_("Specify the value[s] that students must select for this MRQ to be considered correct."),
        scope=Scope.content,
        list_values_provider=QuestionnaireAbstractBlock.choice_values_provider,
        list_style='set',  # Underered, unique items. Affects the UI editor.
        default=[],
    )
    ignored_choices = List(
        display_name=_("Ignored Choices"),
        help=_(
            "Specify the value[s] that are neither correct nor incorrect. "
            "Any values not listed as required or ignored will be considered wrong."
        ),
        scope=Scope.content,
        list_values_provider=QuestionnaireAbstractBlock.choice_values_provider,
        list_style='set',  # Underered, unique items. Affects the UI editor.
        default=[],
    )
    message = String(
        display_name=_("Message"),
        help=_("General feedback provided when submitting"),
        scope=Scope.content,
        default=""
    )
    hide_results = Boolean(display_name="Hide results", scope=Scope.content, default=False)
    editable_fields = (
        'question', 'required_choices', 'ignored_choices', 'message', 'display_name',
        'show_title', 'weight', 'hide_results',
    )

    def describe_choice_correctness(self, choice_value):
        if choice_value in self.required_choices:
            return self._(u"Required")
        elif choice_value in self.ignored_choices:
            return self._(u"Ignored")
        return self._(u"Not Acceptable")

    def get_results(self, previous_result):
        """
        Get the results a student has already submitted.
        """
        result = self.calculate_results(previous_result['submissions'])
        result['completed'] = True
        return result

    def get_last_result(self):
        if self.student_choices:
            return self.get_results({'submissions': self.student_choices})
        else:
            return {}

    def submit(self, submissions):
        log.debug(u'Received MRQ submissions: "%s"', submissions)

        result = self.calculate_results(submissions)
        self.student_choices = submissions

        log.debug(u'MRQ submissions result: %s', result)
        return result

    def calculate_results(self, submissions):
        score = 0
        results = []
        tips = None

        if not self.hide_results:
            tips = self.get_tips()

        for choice in self.custom_choices:
            choice_completed = True
            choice_tips_html = []
            choice_selected = choice.value in submissions

            if choice.value in self.required_choices:
                if not choice_selected:
                    choice_completed = False
            elif choice_selected and choice.value not in self.ignored_choices:
                choice_completed = False

            if choice_completed:
                score += 1

            choice_result = {
                'value': choice.value,
                'selected': choice_selected,
                'content': choice.content
            }
            # Only include tips/results in returned response if we want to display them
            if not self.hide_results:
                # choice_tips_html list is being set only when 'self.hide_results' is False, to optimize,
                # execute the loop only when 'self.hide_results' is set to False
                for tip in tips:
                    if choice.value in tip.values:
                        choice_tips_html.append(tip.render('mentoring_view').content)
                        break

                loader = ResourceLoader(__name__)
                choice_result['completed'] = choice_completed
                choice_result['tips'] = loader.render_template('templates/html/tip_choice_group.html', {
                    'tips_html': choice_tips_html,
                })

            results.append(choice_result)

        status = 'incorrect' if score <= 0 else 'correct' if score >= len(results) else 'partial'

        if sub_api:
            # Send the answer as a concatenated list to the submissions API
            answer = [choice['content'] for choice in results if choice['selected']]
            sub_api.create_submission(self.student_item_key, ', '.join(answer))

        return {
            'submissions': submissions,
            'status': status,
            'choices': results,
            'message': self.message_formatted,
            'weight': self.weight,
            'score': (float(score) / len(results)) if results else 0,
        }

    def validate_field_data(self, validation, data):
        """
        Validate this block's field data.
        """
        super(MRQBlock, self).validate_field_data(validation, data)

        def add_error(msg):
            validation.add(ValidationMessage(ValidationMessage.ERROR, msg))

        def choice_name(choice_value):
            for choice in self.human_readable_choices:
                if choice["value"] == choice_value:
                    return choice["display_name"]
            return choice_value

        all_values = set(self.all_choice_values)
        required = set(data.required_choices)
        ignored = set(data.ignored_choices)

        if len(required) < len(data.required_choices):
            add_error(self._(u"Duplicate required choices set"))
        if len(ignored) < len(data.ignored_choices):
            add_error(self._(u"Duplicate ignored choices set"))
        for val in required.intersection(ignored):
            add_error(self._(u"A choice is listed as both required and ignored: {}").format(choice_name(val)))
        for val in (required - all_values):
            add_error(self._(u"A choice value listed as required does not exist: {}").format(choice_name(val)))
        for val in (ignored - all_values):
            add_error(self._(u"A choice value listed as ignored does not exist: {}").format(choice_name(val)))

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course Block API.
        """
        return {
            'id': self.name,
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name,
            'title': self.display_name,
            'type': self.CATEGORY,
            'weight': self.weight,
            'question': self.expand_static_url(self.question),
            'message': self.message,
            'choices': [
                {'value': choice['value'], 'content': self.expand_static_url(choice['display_name'])}
                for choice in self.human_readable_choices
            ],
            'hide_results': self.hide_results,
            'tips': [tip.student_view_data() for tip in
                     self.get_tips()] if not self.hide_results else [],
        }



import json

import six
import webob
from lazy import lazy
from xblock.core import XBlock
from xblock.fields import UNIQUE_ID, Boolean, Float, Scope, String
from xblock.fragment import Fragment
from xblockutils.helpers import child_isinstance
from xblockutils.resources import ResourceLoader

from problem_builder.tests.unit.utils import DateTimeEncoder

loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text


def _normalize_id(key):
    """
    Helper method to normalize a key to avoid issues where some keys have version/branch and others don't.
    e.g. self.scope_ids.usage_id != self.runtime.get_block(self.scope_ids.usage_id).scope_ids.usage_id
    """
    if hasattr(key, "for_branch"):
        key = key.for_branch(None)
    if hasattr(key, "for_version"):
        key = key.for_version(None)
    return key


class XBlockWithTranslationServiceMixin(object):
    """
    Mixin providing access to i18n service
    """
    def _(self, text):
        """ Translate text """
        return self.runtime.service(self, "i18n").ugettext(text)


class EnumerableChildMixin(XBlockWithTranslationServiceMixin):
    CAPTION = _(u"Child")

    show_title = Boolean(
        display_name=_("Show title"),
        help=_("Display the title?"),
        default=True,
        scope=Scope.content
    )

    @lazy
    def siblings(self):
        # TODO: It might make sense to provide a default
        # implementation here that just returns normalized ID's of the
        # parent's children.
        raise NotImplementedError("Should be overridden in child class")

    @lazy
    def step_number(self):
        return list(self.siblings).index(_normalize_id(self.scope_ids.usage_id)) + 1

    @lazy
    def lonely_child(self):
        if _normalize_id(self.scope_ids.usage_id) not in self.siblings:
            message = u"{child_caption}'s parent should contain {child_caption}".format(child_caption=self.CAPTION)
            raise ValueError(message, self, self.siblings)
        return len(self.siblings) == 1

    @property
    def display_name_with_default(self):
        """ Get the title/display_name of this question. """
        if self.display_name:
            return self.display_name
        if not self.lonely_child:
            return self._(u"{child_caption} {number}").format(
                child_caption=self.CAPTION, number=self.step_number
            )
        return self._(self.CAPTION)


class StepParentMixin(object):
    """
    An XBlock mixin for a parent block containing Step children
    """

    @lazy
    def step_ids(self):
        """
        Get the usage_ids of all of this XBlock's children that are "Steps"
        """
        return [
            _normalize_id(child_id) for child_id in self.children if child_isinstance(self, child_id, QuestionMixin)
        ]

    @lazy
    def steps(self):
        """ Get the step children of this block, cached if possible. """
        return [self.runtime.get_block(child_id) for child_id in self.step_ids]


class MessageParentMixin(object):
    """
    An XBlock mixin for a parent block containing MentoringMessageBlock children
    """

    def get_message_content(self, message_type, or_default=False):
        from problem_builder.message import MentoringMessageBlock  # Import here to avoid circular dependency
        for child_id in self.children:
            if child_isinstance(self, child_id, MentoringMessageBlock):
                child = self.runtime.get_block(child_id)
                if child.type == message_type:
                    content = child.content
                    if getattr(self.runtime, 'replace_jump_to_id_urls', None) is not None:
                        content = self.runtime.replace_jump_to_id_urls(content)
                    return content
        if or_default:
            # Return the default value since no custom message is set.
            # Note the WYSIWYG editor usually wraps the .content HTML in a <p> tag so we do the same here.
            return '<p>{}</p>'.format(MentoringMessageBlock.MESSAGE_TYPES[message_type]['default'])


class QuestionMixin(EnumerableChildMixin):
    """
    An XBlock mixin for a child block that is a "Step".

    A step is a question that the user can answer (as opposed to a read-only child).
    """
    CAPTION = _(u"Question")

    has_author_view = True

    # Fields:
    name = String(
        display_name=_("Question ID (name)"),
        help=_("The ID of this question (required). Should be unique within this mentoring component."),
        default=UNIQUE_ID,
        scope=Scope.settings,  # Must be scope.settings, or the unique ID will change every time this block is edited
    )
    display_name = String(
        display_name=_("Question title"),
        help=_('Leave blank to use the default ("Question 1", "Question 2", etc.)'),
        default="",  # Blank will use 'Question x' - see display_name_with_default
        scope=Scope.content
    )
    weight = Float(
        display_name=_("Weight"),
        help=_("Defines the maximum total grade of this question."),
        default=1,
        scope=Scope.content,
        enforce_type=True
    )

    @lazy
    def siblings(self):
        return self.get_parent().step_ids

    def author_view(self, context):
        context = context.copy() if context else {}
        context['hide_header'] = True
        return self.mentoring_view(context)

    def author_preview_view(self, context):
        context = context.copy() if context else {}
        context['hide_header'] = True
        return self.student_view(context)


class NoSettingsMixin(object):
    """ Mixin for an XBlock that has no settings """

    def studio_view(self, _context=None):
        """ Studio View """
        return Fragment(u'<p>{}</p>'.format(self._("This XBlock does not have any settings.")))


class StudentViewUserStateMixin(object):
    """
    Mixin to provide student_view_user_state view.

    To prevent unnecessary overloading of the build_user_state_data method,
    you may specify `USER_STATE_FIELDS` to customise build_user_state_data
    and student_view_user_state output.
    """
    NESTED_BLOCKS_KEY = "components"
    INCLUDE_SCOPES = (Scope.user_state, Scope.user_info, Scope.preferences)
    USER_STATE_FIELDS = []

    def transforms(self):
        """
        Return a dict where keys are fields to transform, and values are
        transform functions that accept a value to to transform as the
        only argument.
        """
        return {}

    def build_user_state_data(self, context=None):
        """
        Returns a dictionary of the student data of this XBlock,
        retrievable from the student_view_user_state XBlock handler.
        """

        result = {}
        transforms = self.transforms()
        for _, field in six.iteritems(self.fields):
            # Only insert fields if their scopes and field names match
            if field.scope in self.INCLUDE_SCOPES and field.name in self.USER_STATE_FIELDS:
                transformer = transforms.get(field.name, lambda value: value)
                result[field.name] = transformer(field.read_from(self))

        if getattr(self, "has_children", False):
            components = {}
            for child_id in self.children:
                child = self.runtime.get_block(child_id)
                if hasattr(child, 'build_user_state_data'):
                    components[str(child_id)] = child.build_user_state_data(context)

            result[self.NESTED_BLOCKS_KEY] = components

        return result

    @XBlock.handler
    def student_view_user_state(self, context=None, suffix=''):
        """
        Returns a JSON representation of the student data of this XBlock.
        """
        result = self.build_user_state_data(context)
        json_result = json.dumps(result, cls=DateTimeEncoder)

        return webob.response.Response(
            body=json_result.encode('utf-8'),
            content_type='application/json'
        )


class StudentViewUserStateResultsTransformerMixin(object):
    """
    A convenient way for MentoringBlock and MentoringStepBlock to share
    student_results transform code.
    Needs to be used alongside StudentViewUserStateMixin.
    """
    def transforms(self):
        return {
            'student_results': self.transform_student_results
        }

    def transform_student_results(self, student_results):
        """
        Remove tips, since they are already in student_view_data.
        """
        for _name, current_student_results in student_results:
            for choice in current_student_results.get('choices', []):
                self.delete_key(choice, 'tips')
            self.delete_key(current_student_results, 'tips')

        return student_results

    def delete_key(self, dictionary, key):
        """
        Safely delete `key` from `dictionary`.
        """
        try:
            del dictionary[key]
        except KeyError:
            pass
        return dictionary


class ExpandStaticURLMixin(object):

    def expand_static_url(self, text):
        """
        This is required to make URLs like '/static/dnd-test-image.png' work (note: that is the
        only portable URL format for static files that works across export/import and reruns).
        This method is unfortunately a bit hackish since XBlock does not provide a low-level API
        for this.
        """
        if not text:
            return text

        if hasattr(self.runtime, 'replace_urls'):
            text = self.runtime.replace_urls(text)
        elif hasattr(self.runtime, 'course_id'):
            # edX Studio uses a different runtime for 'studio_view' than 'student_view',
            # and the 'studio_view' runtime doesn't provide the replace_urls API.
            try:
                from static_replace import replace_static_urls  # pylint: disable=import-error
                text = replace_static_urls(text, course_id=self.runtime.course_id)
            except ImportError:
                pass
        return text



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import math
import re
import time

from mock import Mock, patch
from selenium.common.exceptions import NoSuchElementException
from xblockutils.base_test import SeleniumXBlockTest

from problem_builder.instructor_tool import PAGE_SIZE, InstructorToolBlock


class MockTasksModule(object):
    """Mock for the tasks module, which can only be meaningfully import in the LMS."""

    def __init__(self, successful=True, display_data=[]):
        self.export_data = Mock()
        async_result = self.export_data.async_result
        async_result.ready.side_effect = [False, False, True, True]
        async_result.id = "export_task_id"
        async_result.successful.return_value = successful
        if successful:
            async_result.result = dict(
                error=None,
                report_filename='/file/report.csv',
                start_timestamp=time.time(),
                generation_time_s=23.4,
                display_data=display_data
            )
        else:
            async_result.result = 'error'
        self.export_data.AsyncResult.return_value = async_result
        self.export_data.delay.return_value = async_result


class MockInstructorTaskModelsModule(object):

    def __init__(self):
        self.ReportStore = Mock()
        self.ReportStore.from_config.return_value.links_for.return_value = [
            ('/file/report.csv', '/url/report.csv')
        ]


class InstructorToolTest(SeleniumXBlockTest):

    def setUp(self):
        super(InstructorToolTest, self).setUp()
        self.set_scenario_xml("""
        <vertical_demo>
          <pb-instructor-tool url_name="data_export"/>
          <problem-builder>
            <pb-answer name="answer" question="Is this a long long long long long long long long long long question?" />
          </problem-builder>
        </vertical_demo>
        """)

    def test_students_dont_see_interface(self):
        data_export = self.go_to_view()
        self.assertIn('This interface can only be used by course staff.', data_export.text)

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=True),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_export_field_container_width(self):
        instructor_tool = self.go_to_view()

        export_field_container = instructor_tool.find_element_by_class_name('data-export-field-container')
        parent_div = export_field_container.find_element_by_xpath('..')

        export_field_container_width = export_field_container.size['width']
        parent_div_width = parent_div.size['width']

        self.assertTrue(export_field_container_width <= math.ceil(0.43 * parent_div_width))

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=True),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_root_block_select_width(self):
        instructor_tool = self.go_to_view()

        root_block_select = instructor_tool.find_element_by_name('root_block_id')
        parent_div = root_block_select.find_element_by_xpath('../..')

        root_block_select_width = root_block_select.size['width']
        parent_div_width = parent_div.size['width']

        self.assertTrue(root_block_select_width <= math.ceil(0.55 * parent_div_width))

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=True),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_data_export_delete(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        status_area = instructor_tool.find_element_by_class_name('data-export-status')
        download_button = instructor_tool.find_element_by_class_name('data-export-download')
        cancel_button = instructor_tool.find_element_by_class_name('data-export-cancel')
        delete_button = instructor_tool.find_element_by_class_name('data-export-delete')

        start_button.click()

        self.wait_until_visible(result_block)
        self.wait_until_visible(delete_button)

        delete_button.click()

        self.wait_until_hidden(result_block)
        self.wait_until_hidden(delete_button)

        self.assertTrue(start_button.is_enabled())
        self.assertEqual('', status_area.text)
        self.assertFalse(download_button.is_displayed())
        self.assertFalse(cancel_button.is_displayed())

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=True),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_data_export_success(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        info_area = instructor_tool.find_element_by_class_name('data-export-info')
        status_area = instructor_tool.find_element_by_class_name('data-export-status')
        download_button = instructor_tool.find_element_by_class_name('data-export-download')
        cancel_button = instructor_tool.find_element_by_class_name('data-export-cancel')
        delete_button = instructor_tool.find_element_by_class_name('data-export-delete')

        start_button.click()

        self.wait_until_hidden(start_button)
        self.wait_until_hidden(result_block)
        self.wait_until_hidden(download_button)
        self.wait_until_visible(cancel_button)
        self.wait_until_hidden(delete_button)
        self.assertIn('The report is currently being generated', status_area.text)

        self.wait_until_visible(start_button)
        self.wait_until_visible(result_block)
        self.wait_until_visible(download_button)
        self.wait_until_hidden(cancel_button)
        self.wait_until_visible(delete_button)
        time.sleep(1)

        for column in [
                'Section', 'Subsection', 'Unit',
                'Type', 'Question', 'Answer', 'Username'
        ]:
            self.assertIn(column, result_block.text)
        self.assertIn('Results retrieved on', info_area.text)
        self.assertEqual('', status_area.text)

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=False),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_data_export_error(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        info_area = instructor_tool.find_element_by_class_name('data-export-info')
        status_area = instructor_tool.find_element_by_class_name('data-export-status')
        download_button = instructor_tool.find_element_by_class_name('data-export-download')
        cancel_button = instructor_tool.find_element_by_class_name('data-export-cancel')
        delete_button = instructor_tool.find_element_by_class_name('data-export-delete')

        start_button.click()

        self.wait_until_hidden(start_button)
        self.wait_until_hidden(result_block)
        self.wait_until_visible(cancel_button)
        self.wait_until_hidden(download_button)
        self.wait_until_hidden(delete_button)
        self.assertIn('The report is currently being generated', status_area.text)

        self.wait_until_visible(start_button)
        self.wait_until_hidden(cancel_button)
        self.wait_until_visible(delete_button)
        self.assertFalse(result_block.is_displayed())
        self.assertEqual('', info_area.text)
        self.assertIn('Data export failed. Reason:', status_area.text)

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(successful=True),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_pagination_no_results(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        first_page_button = instructor_tool.find_element_by_id('first-page')
        prev_page_button = instructor_tool.find_element_by_id('prev-page')
        next_page_button = instructor_tool.find_element_by_id('next-page')
        last_page_button = instructor_tool.find_element_by_id('last-page')
        current_page_info = instructor_tool.find_element_by_id('current-page')
        total_pages_info = instructor_tool.find_element_by_id('total-pages')

        start_button.click()

        self.wait_until_visible(result_block)
        time.sleep(1)  # Allow some time for result block to fully fade in

        self.assertFalse(first_page_button.is_enabled())
        self.assertFalse(prev_page_button.is_enabled())
        self.assertFalse(next_page_button.is_enabled())
        self.assertFalse(last_page_button.is_enabled())

        self.assertEqual('0', current_page_info.text)
        self.assertEqual('0', total_pages_info.text)

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(
            successful=True, display_data=[[
                'Test section', 'Test subsection', 'Test unit',
                'Test type', 'Test question', 'Test answer', 'Test username'
            ]]),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_pagination_single_result(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        first_page_button = instructor_tool.find_element_by_id('first-page')
        prev_page_button = instructor_tool.find_element_by_id('prev-page')
        next_page_button = instructor_tool.find_element_by_id('next-page')
        last_page_button = instructor_tool.find_element_by_id('last-page')
        current_page_info = instructor_tool.find_element_by_id('current-page')
        total_pages_info = instructor_tool.find_element_by_id('total-pages')

        start_button.click()

        self.wait_until_visible(result_block)
        time.sleep(1)  # Allow some time for result block to fully fade in

        for contents in [
                'Test section', 'Test subsection', 'Test unit',
                'Test type', 'Test question', 'Test answer', 'Test username'
        ]:
            self.assertIn(contents, result_block.text)

        self.assertFalse(first_page_button.is_enabled())
        self.assertFalse(prev_page_button.is_enabled())
        self.assertFalse(next_page_button.is_enabled())
        self.assertFalse(last_page_button.is_enabled())

        self.assertEqual('1', current_page_info.text)
        self.assertEqual('1', total_pages_info.text)

    @patch.dict('sys.modules', {
        'problem_builder.tasks': MockTasksModule(
            successful=True, display_data=[[
                'Test section', 'Test subsection', 'Test unit',
                'Test type', 'Test question', 'Test answer', 'Test username'
            ] for _ in range(PAGE_SIZE*3)]),
        'lms': True,
        'lms.djangoapps': True,
        'lms.djangoapps.instructor_task': True,
        'lms.djangoapps.instructor_task.models': MockInstructorTaskModelsModule(),
    })
    @patch.object(InstructorToolBlock, 'user_is_staff', Mock(return_value=True))
    def test_pagination_multiple_results(self):
        instructor_tool = self.go_to_view()
        start_button = instructor_tool.find_element_by_class_name('data-export-start')
        result_block = instructor_tool.find_element_by_class_name('data-export-results')
        first_page_button = instructor_tool.find_element_by_id('first-page')
        prev_page_button = instructor_tool.find_element_by_id('prev-page')
        next_page_button = instructor_tool.find_element_by_id('next-page')
        last_page_button = instructor_tool.find_element_by_id('last-page')
        current_page_info = instructor_tool.find_element_by_id('current-page')
        total_pages_info = instructor_tool.find_element_by_id('total-pages')

        start_button.click()

        self.wait_until_visible(result_block)
        time.sleep(1)  # Allow some time for result block to fully fade in

        for contents in [
                'Test section', 'Test subsection', 'Test unit',
                'Test type', 'Test question', 'Test answer', 'Test username'
        ]:
            occurrences = re.findall(contents, result_block.text)
            self.assertEqual(len(occurrences), PAGE_SIZE)

        self.assertFalse(first_page_button.is_enabled())
        self.assertFalse(prev_page_button.is_enabled())
        self.assertTrue(next_page_button.is_enabled())
        self.assertTrue(last_page_button.is_enabled())

        self.assertEqual('1', current_page_info.text)
        self.assertEqual('3', total_pages_info.text)

        # Test behavior of pagination controls

        # - "Next" button

        next_page_button.click()  # Navigate to second page

        self.assertTrue(first_page_button.is_enabled())
        self.assertTrue(prev_page_button.is_enabled())
        self.assertTrue(next_page_button.is_enabled())
        self.assertTrue(last_page_button.is_enabled())

        self.assertEqual('2', current_page_info.text)

        next_page_button.click()  # Navigate to third page

        self.assertTrue(first_page_button.is_enabled())
        self.assertTrue(prev_page_button.is_enabled())
        self.assertFalse(next_page_button.is_enabled())
        self.assertFalse(last_page_button.is_enabled())

        self.assertEqual('3', current_page_info.text)

        # - "Prev" button

        prev_page_button.click()  # Navigate to second page

        self.assertTrue(first_page_button.is_enabled())
        self.assertTrue(prev_page_button.is_enabled())
        self.assertTrue(next_page_button.is_enabled())
        self.assertTrue(last_page_button.is_enabled())

        self.assertEqual('2', current_page_info.text)

        prev_page_button.click()  # Navigate to first page

        self.assertFalse(first_page_button.is_enabled())
        self.assertFalse(prev_page_button.is_enabled())
        self.assertTrue(next_page_button.is_enabled())
        self.assertTrue(last_page_button.is_enabled())

        self.assertEqual('1', current_page_info.text)

        # - "Last" button

        last_page_button.click()  # Navigate to last page

        self.assertTrue(first_page_button.is_enabled())
        self.assertTrue(prev_page_button.is_enabled())
        self.assertFalse(next_page_button.is_enabled())
        self.assertFalse(last_page_button.is_enabled())

        self.assertEqual('3', current_page_info.text)

        # - "First" button

        first_page_button.click()  # Navigate to first page

        self.assertFalse(first_page_button.is_enabled())
        self.assertFalse(prev_page_button.is_enabled())
        self.assertTrue(next_page_button.is_enabled())
        self.assertTrue(last_page_button.is_enabled())

        self.assertEqual('1', current_page_info.text)

    def test_non_staff_disabled(self):
        instructor_tool = self.go_to_view()
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-start'
        )
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-cancel'
        )
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-download'
        )
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-delete'
        )
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-results'
        )
        self.assertRaises(
            NoSuchElementException, instructor_tool.find_element_by_class_name, 'data-export-status'
        )



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
import ddt

from .base_test import ProblemBuilderBaseTest

COMPLETED, INCOMPLETE, MAX_REACHED = "completed", "incomplete", "max_attempts_reached"
MESSAGES = {
    COMPLETED: u"Great job! (completed message)",
    INCOMPLETE: u"Not quite! You can try again, though. (incomplete message)",
    MAX_REACHED: (
        u"Sorry, you have used up all of your allowed submissions. (max_attempts_reached message)"
    ),
}


@ddt.ddt
class MessagesTest(ProblemBuilderBaseTest):
    """
    Test the various types of message that can be added to a problem.
    """
    def expect_message(self, msg_type, mentoring):
        """
        Assert that the message of the specified type is shown to the user.
        """
        messages_element = mentoring.find_element_by_css_selector('.messages')
        if msg_type is None:
            self.assertFalse(messages_element.is_displayed())
        else:
            self.assertTrue(messages_element.is_displayed())
            message_text = messages_element.text.strip()
            self.assertTrue(message_text.startswith("FEEDBACK"))
            message_text = message_text[8:].lstrip()
            self.assertEqual(MESSAGES[msg_type], message_text)

    @ddt.data(
        ("One", COMPLETED),
        ("Two", COMPLETED),
        ("I don't understand", MAX_REACHED),
    )
    @ddt.unpack
    def test_one_shot(self, choice_text, expected_message_type):
        """
        Test a question that has max_attempts set to 1
        """
        mentoring = self.load_scenario("messages.xml", {"max_attempts": 1})
        self.expect_message(None, mentoring)
        self.click_choice(mentoring, choice_text)
        self.click_submit(mentoring)
        self.expect_message(expected_message_type, mentoring)

    @ddt.data(
        (2, "One", COMPLETED),
        (2, "I don't understand", MAX_REACHED),
        (0, "I don't understand", INCOMPLETE),
        (10, "I don't understand", INCOMPLETE),
    )
    @ddt.unpack
    def test_retry(self, max_attempts, choice_text, expected_message_type):
        """
        Test submitting a wrong answer, seeing a message, then submitting another answer.
        In each case, max_attempts is not 1.
        """
        mentoring = self.load_scenario("messages.xml", {"max_attempts": max_attempts})
        # First, there should be no message.
        self.expect_message(None, mentoring)
        # Let's get the question wrong:
        self.click_choice(mentoring, "I don't understand")
        self.click_submit(mentoring)
        # We should now see the INCOMPLETE message:
        self.expect_message(INCOMPLETE, mentoring)

        # Now, answer as directed and expect the given message:
        self.click_choice(mentoring, "One")  # Make sure we change our choice so we will be able to submit
        self.click_choice(mentoring, choice_text)
        self.click_submit(mentoring)
        self.expect_message(expected_message_type, mentoring)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

import uuid

import six
from lxml import etree
from xblock.core import XBlock
from xblock.fields import Scope, String
from xblock.fragment import Fragment
from xblock.validation import ValidationMessage
from xblockutils.studio_editable import (StudioEditableXBlockMixin,
                                         XBlockWithPreviewMixin)

from problem_builder.mixins import (ExpandStaticURLMixin,
                                    StudentViewUserStateMixin,
                                    XBlockWithTranslationServiceMixin)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text

# Classes ###########################################################


@XBlock.needs("i18n")
class ChoiceBlock(
    StudioEditableXBlockMixin, XBlockWithPreviewMixin, XBlockWithTranslationServiceMixin, StudentViewUserStateMixin,
    XBlock, ExpandStaticURLMixin
):
    """
    Custom choice of an answer for a MCQ/MRQ
    """
    value = String(
        display_name=_("Value"),
        help=_("Value of the choice when selected. Should be unique. Generally you do not need to edit this."),
        scope=Scope.content,
        default="",
    )
    content = String(
        display_name=_("Choice Text"),
        help=_("Human-readable version of the choice value"),
        scope=Scope.content,
        default="",
    )
    editable_fields = ('content', 'value')

    @property
    def display_name_with_default(self):
        try:
            status = self.get_parent().describe_choice_correctness(self.value)
        except Exception:
            status = self._(u"Out of Context")  # Parent block should implement describe_choice_correctness()
        return self._(u"Choice ({status})").format(status=status)

    def student_view_data(self, context=None):
        """
        Returns a JSON representation of the student_view of this XBlock,
        retrievable from the Course Block API.
        """
        # display_name_with_default gives out correctness - not adding it here
        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.expand_static_url(self._(u"Choice ({content})").format(content=self.content)),
            'value': self.value,
            'content': self.expand_static_url(self.content),
        }

    def mentoring_view(self, context=None):
        """ Render this choice string within a mentoring block question. """
        return Fragment(u'<span class="choice-text">{}</span>'.format(self.content))

    def student_view(self, context=None):
        """ Normal view of this XBlock, identical to mentoring_view """
        return self.mentoring_view(context)

    def validate_field_data(self, validation, data):
        """
        Validate this block's field data.
        """
        super(ChoiceBlock, self).validate_field_data(validation, data)

        def add_error(msg):
            validation.add(ValidationMessage(ValidationMessage.ERROR, msg))

        if not data.value.strip():
            add_error(self._(u"No value set. This choice will not work correctly."))
        if not data.content.strip():
            add_error(self._(u"No choice text set yet."))

    def validate(self):
        """
        Validates the state of this XBlock.
        """
        validation = super(ChoiceBlock, self).validate()
        if self.get_parent().all_choice_values.count(self.value) > 1:
            validation.add(
                ValidationMessage(ValidationMessage.ERROR, self._(
                    u"This choice has a non-unique ID and won't work properly. "
                    "This can happen if you duplicate a choice rather than use the Add Choice button."
                ))
            )
        print(self.get_parent().all_choice_values)
        return validation

    @classmethod
    def get_template(cls, template_id):
        """
        Used to interact with Studio's create_xblock method to instantiate pre-defined templates.
        """
        # Generate a random 'value' value. We can't just use default=UNIQUE_ID on the field,
        # because that doesn't work properly with import/export, re-run, or duplicating the block
        if template_id == 'studio_default':
            return {'data': {'value': uuid.uuid4().hex[:7]}}
        return {'metadata': {}, 'data': {}}

    @classmethod
    def parse_xml(cls, node, runtime, keys, id_generator):
        """
        Construct this XBlock from the given XML node.
        """
        block = runtime.construct_xblock_from_class(cls, keys)
        for field_name in ('value', 'content'):
            if field_name in node.attrib:
                setattr(block, field_name, node.attrib[field_name])

        # HTML content:
        block.content = six.text_type(node.text or u"")
        for child in node:
            block.content += etree.tostring(child, encoding='unicode')

        return block



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
Dashboard: Summarize the results of self-assessments done by a student with Problem Builder MCQ
blocks.

The author of this block specifies a list of Problem Builder blocks containing MCQs. This block
will then display a table summarizing the values that the student chose for each of those MCQs.
"""
# Imports ###########################################################

import ast
import json
import logging
import operator as op

import six
from django.template.defaultfilters import floatformat
from lazy import lazy
from xblock.core import XBlock
from xblock.fields import Boolean, Dict, List, Scope, String
from xblock.fragment import Fragment
from xblock.validation import ValidationMessage
from xblockutils.helpers import child_isinstance
from xblockutils.resources import ResourceLoader
from xblockutils.studio_editable import StudioEditableXBlockMixin

from .dashboard_visual import DashboardVisualData
from .mcq import MCQBlock
from .sub_api import sub_api

# Globals ###########################################################

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)


def _(text):
    """ A no-op to mark strings that we need to translate """
    return text

# Classes ###########################################################


class ExportMixin(object):
    """
    Used by blocks which need to provide a downloadable export.
    """
    def _get_user_full_name(self):
        """
        Get the full name of the current user, for the downloadable report.
        """
        user_service = self.runtime.service(self, 'user')
        if user_service:
            return user_service.get_current_user().full_name
        return ""

    def _get_course_name(self):
        """
        Get the name of the current course, for the downloadable report.
        """
        try:
            course_key = self.scope_ids.usage_id.course_key
        except AttributeError:
            return ""  # We are not in an edX runtime
        try:
            course_root_key = course_key.make_usage_key('course', 'course')
            return self.runtime.get_block(course_root_key).display_name
        except Exception:  # ItemNotFoundError most likely, but we can't import that exception in non-edX environments
            # We may be on old mongo:
            try:
                course_root_key = course_key.make_usage_key('course', course_key.run)
                return self.runtime.get_block(course_root_key).display_name
            except Exception:
                return ""


class ColorRule(object):
    """
    A rule used to conditionally set colors

    >>> rule1 = ColorRule("3 < x <= 5", color_str="#ff0000")
    >>> rule1.matches(2)
    False
    >>> rule1.matches(4)
    True
    """
    def __init__(self, rule_str, color_str):
        """
        Instantiate a ColorRule with the given rule expression string and color value.
        """
        try:
            self._rule_parsed = ast.parse(rule_str, mode='eval').body
            # Once it's been parsed, also try evaluating it with a test value:
            self._safe_eval_expression(self._rule_parsed, x=0)
        except (TypeError, SyntaxError) as e:
            raise ValueError("Invalid Expression: {}".format(e))
        except ZeroDivisionError:
            pass  # This may depend on the value of 'x' which we set to zero but don't know yet.
        self.color_str = color_str

    def matches(self, x):
        """ Does this rule apply for the value x? """
        try:
            return bool(self._safe_eval_expression(self._rule_parsed, x))
        except ZeroDivisionError:
            return False

    @staticmethod
    def _safe_eval_expression(expr, x=0):
        """
        Safely evaluate a mathematical or boolean expression involving the value x

        >>> _safe_eval_expression('2*x', x=3)
        6
        >>> _safe_eval_expression('x >= 0 and x < 2', x=3)
        False

        We use python syntax for the expressions, so the parsing and evaluation is mostly done
        using Python's built-in asbstract syntax trees and operator implementations.

        The expression can only contain: numbers, mathematical operators, boolean operators,
        comparisons, and a placeholder variable called "x"
        """
        # supported operators:
        operators = {
            # Allow +, -, *, /, %, negative:
            ast.Add: op.add, ast.Sub: op.sub, ast.Mult: op.mul, ast.Div: op.truediv, ast.Mod: op.mod, ast.USub: op.neg,
            # Allow comparison:
            ast.Eq: op.eq, ast.NotEq: op.ne, ast.Lt: op.lt, ast.LtE: op.le, ast.Gt: op.gt, ast.GtE: op.ge,
        }

        def eval_(node):
            """ Recursive evaluation of syntax tree node 'node' """
            if isinstance(node, ast.Num):  # <number>
                return node.n
            elif isinstance(node, ast.BinOp):  # <left> <operator> <right>
                return operators[type(node.op)](eval_(node.left), eval_(node.right))
            elif isinstance(node, ast.UnaryOp):  # <operator> <operand> e.g., -1
                return operators[type(node.op)](eval_(node.operand))
            elif isinstance(node, ast.Name) and node.id == "x":
                return x
            elif isinstance(node, ast.BoolOp):  # Boolean operator: either "and" or "or" with two or more values
                if type(node.op) == ast.And:
                    return all(eval_(val) for val in node.values)
                else:  # Or:
                    for val in node.values:
                        result = eval_(val)
                        if result:
                            return result
                    return result  # or returns the final value even if it's falsy
            elif isinstance(node, ast.Compare):  # A comparison expression, e.g. "3 > 2" or "5 < x < 10"
                left = eval_(node.left)
                for comparison_op, right_expr in zip(node.ops, node.comparators):
                    right = eval_(right_expr)
                    if not operators[type(comparison_op)](left, right):
                        return False
                    left = right
                return True
            else:
                raise TypeError(node)

        if not isinstance(expr, ast.AST):
            expr = ast.parse(expr, mode='eval').body
        return eval_(expr)


class InvalidUrlName(ValueError):
    """ Exception raised by DashboardBlock.get_mentoring_blocks() if a url_name is invalid """
    pass


@XBlock.needs("i18n")
@XBlock.wants("user")
class DashboardBlock(StudioEditableXBlockMixin, ExportMixin, XBlock):
    """
    A block to summarize self-assessment results.
    """
    display_name = String(
        display_name=_("Display Name"),
        help=_("Display name for this module"),
        scope=Scope.settings,
        default=_('Self-Assessment Summary'),
    )
    mentoring_ids = List(
        display_name=_("Mentoring Blocks"),
        help=_(
            "This should be an ordered list of the url_names of each mentoring block whose multiple choice question "
            "values are to be shown on this dashboard. The list should be in JSON format. Example: {example_here}"
        ).format(example_here='["2754b8afc03a439693b9887b6f1d9e36", "215028f7df3d4c68b14fb5fea4da7053"]'),
        scope=Scope.settings,
    )
    exclude_questions = Dict(
        display_name=_("Questions to be hidden"),
        help=_(
            "Optional rules to exclude specific questions both from displaying in dashboard and from the calculated "
            "average. Rules must start with the url_name of a mentoring block, followed by list of question numbers "
            "to exclude. Rule set must be in JSON format. Question numbers are one-based (the first question being "
            "number 1). Must be in JSON format. Examples: {examples_here}"
        ).format(
            examples_here='{"2754b8afc03a439693b9887b6f1d9e36":[1,2], "215028f7df3d4c68b14fb5fea4da7053":[1,5]}'
        ),
        scope=Scope.content,
        multiline_editor=True,
        resettable_editor=False,
    )
    color_rules = String(
        display_name=_("Color Coding Rules"),
        help=_(
            "Optional rules to assign colors to possible answer values and average values. "
            "One rule per line. First matching rule will be used. Light colors are recommended. "
            "Examples: {examples_here}"
        ).format(examples_here='"1: LightCoral", "0 <= x < 5: LightBlue", "LightGreen"'),
        scope=Scope.content,
        default="",
        multiline_editor=True,
        resettable_editor=False,
    )
    visual_rules = String(
        display_name=_("Visual Representation"),
        default="",
        help=_("Optional: Enter the JSON configuration of the visual representation desired (Advanced)."),
        scope=Scope.content,
        multiline_editor=True,
        resettable_editor=False,
    )
    visual_title = String(
        display_name=_("Visual Representation Title"),
        default=_("Visual Representation"),
        help=_("This text is not displayed visually but is exposed to screen reader users who may not see the image."),
        scope=Scope.content,
    )
    visual_desc = String(
        display_name=_("Visual Repr. Description"),
        default=_("The data represented in this image is available in the tables below."),
        help=_(
            "This longer description is not displayed visually but is exposed to screen reader "
            "users who may not see the image."
        ),
        scope=Scope.content,
    )
    average_labels = Dict(
        display_name=_("Label for average value"),
        help=_(
            "This settings allows overriding label for the calculated average per mentoring block. Must be in JSON "
            "format. Examples: {examples_here}."
        ).format(
            examples_here='{"2754b8afc03a439693b9887b6f1d9e36": "Avg.", "215028f7df3d4c68b14fb5fea4da7053": "Mean"}'
        ),
        scope=Scope.content,
    )
    show_numbers = Boolean(
        display_name=_("Display values"),
        default=True,
        help=_("Toggles if numeric values are displayed"),
        scope=Scope.content
    )
    header_html = String(
        display_name=_("Header HTML"),
        default="",
        help=_("Custom text to include at the beginning of the report."),
        multiline_editor="html",
        resettable_editor=False,
        scope=Scope.content,
    )
    footer_html = String(
        display_name=_("Footer HTML"),
        default="",
        help=_("Custom text to include at the end of the report."),
        multiline_editor="html",
        resettable_editor=False,
        scope=Scope.content,
    )

    editable_fields = (
        'display_name', 'mentoring_ids', 'exclude_questions', 'average_labels', 'show_numbers',
        'color_rules', 'visual_rules', 'visual_title', 'visual_desc', 'header_html', 'footer_html',
    )
    css_path = 'public/css/dashboard.css'
    js_path = 'public/js/review_blocks.js'

    def get_mentoring_blocks(self, mentoring_ids, ignore_errors=True):
        """
        Generator returning the specified mentoring blocks, in order.

        Will yield None for every invalid mentoring block ID, or if
        ignore_errors is False, will raise InvalidUrlName.
        """
        for url_name in mentoring_ids:
            try:
                mentoring_id = self.scope_ids.usage_id.course_key.make_usage_key('problem-builder', url_name)
                yield self.runtime.get_block(mentoring_id)
            except Exception:  # Catch-all b/c we could get XBlockNotFoundError, ItemNotFoundError, InvalidKeyError, ...
                # Maybe it's using the deprecated block type "mentoring":
                try:
                    mentoring_id = self.scope_ids.usage_id.course_key.make_usage_key('mentoring', url_name)
                    yield self.runtime.get_block(mentoring_id)
                except Exception:
                    if ignore_errors:
                        yield None
                    else:
                        raise InvalidUrlName(url_name)

    def parse_color_rules_str(self, color_rules_str, ignore_errors=True):
        """
        Parse the color rules. Returns a list of ColorRule objects.

        Color rules are like: "0 < x < 4: red" or "blue" (for a catch-all rule)
        """
        rules = []
        for lineno, line in enumerate(color_rules_str.splitlines()):
            line = line.strip()
            if line:
                try:
                    if ":" in line:
                        condition, value = line.split(':')
                        value = value.strip()
                        if condition.isnumeric():  # A condition just listed as an exact value
                            condition = "x == " + condition
                    else:
                        condition = "1"  # Always true
                        value = line
                    rules.append(ColorRule(condition, value))
                except ValueError:
                    if ignore_errors:
                        continue
                    raise ValueError(
                        _("Invalid color rule on line {line_number}").format(line_number=lineno + 1)
                    )
        return rules

    @lazy
    def color_rules_parsed(self):
        """
        Caching property to get parsed color rules. Returns a list of ColorRule objects.
        """
        return self.parse_color_rules_str(self.color_rules) if self.color_rules else []

    def _get_submission_key(self, usage_key):
        """
        Given the usage_key of an MCQ block, get the dict key needed to look it up with the
        submissions API.
        """
        return dict(
            student_id=self.runtime.anonymous_student_id,
            course_id=six.text_type(usage_key.course_key),
            item_id=six.text_type(usage_key),
            item_type=usage_key.block_type,
        )

    def color_for_value(self, value):
        """ Given a string value, get the color rule that matches, if any """
        if isinstance(value, six.string_types):
            if value.isnumeric():
                value = float(value)
            else:
                return None
        for rule in self.color_rules_parsed:
            if rule.matches(value):
                return rule.color_str
        return None

    def _get_problem_questions(self, mentoring_block):
        """ Generator returning only children of specified block that are MCQs """
        for child_id in mentoring_block.children:
            if child_isinstance(mentoring_block, child_id, MCQBlock):
                yield child_id

    @XBlock.supports("multi_device")  # Mark as mobile-friendly
    def student_view(self, context=None):  # pylint: disable=unused-argument
        """
        Standard view of this XBlock.
        """
        if not self.mentoring_ids:
            return Fragment(u"<h1>{}</h1><p>{}</p>".format(self.display_name, _("Not configured.")))

        blocks = []
        for mentoring_block in self.get_mentoring_blocks(self.mentoring_ids):
            if mentoring_block is None:
                continue
            block = {
                'display_name': mentoring_block.display_name,
                'mcqs': []
            }
            try:
                hide_questions = self.exclude_questions.get(mentoring_block.url_name, [])
            except Exception:  # pylint: disable=broad-except-clause
                log.exception("Cannot parse exclude_questions setting - probably malformed: %s", self.exclude_questions)
                hide_questions = []

            for question_number, child_id in enumerate(self._get_problem_questions(mentoring_block), 1):
                try:
                    if question_number in hide_questions:
                        continue
                except TypeError:
                    log.exception(
                        "Cannot check question number - expected list of ints got: %s",
                        hide_questions
                    )

                # Get the student's submitted answer to this MCQ from the submissions API:
                mcq_block = self.runtime.get_block(child_id)
                mcq_submission_key = self._get_submission_key(child_id)
                try:
                    value = sub_api.get_submissions(mcq_submission_key, limit=1)[0]["answer"]
                except IndexError:
                    value = None

                block['mcqs'].append({
                    "display_name": mcq_block.display_name_with_default,
                    "value": value,
                    "accessible_value": _("Score: {score}").format(score=value) if value else _("No value yet"),
                    "color": self.color_for_value(value) if value is not None else None,
                })
            # If the values are numeric, display an average:
            numeric_values = [
                float(mcq['value']) for mcq in block['mcqs']
                if mcq['value'] is not None and mcq['value'].isnumeric()
            ]
            if numeric_values:
                average_value = sum(numeric_values) / len(numeric_values)
                block['average'] = average_value
                # average block is shown only if average value exists, so accessible text for no data is not required
                block['accessible_average'] = _("Score: {score}").format(
                    score=floatformat(average_value)
                )
                block['average_label'] = self.average_labels.get(mentoring_block.url_name, _("Average"))
                block['has_average'] = True
                block['average_color'] = self.color_for_value(average_value)
            blocks.append(block)

        visual_repr = None
        if self.visual_rules:
            try:
                rules_parsed = json.loads(self.visual_rules)
            except ValueError:
                pass  # JSON errors should be shown as part of validation
            else:
                visual_repr = DashboardVisualData(
                    blocks, rules_parsed, self.color_for_value, self.visual_title, self.visual_desc
                )

        report_template = loader.render_template('templates/html/dashboard_report.html', {
            'title': self.display_name,
            'css': loader.load_unicode(self.css_path),
            'student_name': self._get_user_full_name(),
            'course_name': self._get_course_name(),
        })

        html = loader.render_template('templates/html/dashboard.html', {
            'blocks': blocks,
            'display_name': self.display_name,
            'visual_repr': visual_repr,
            'show_numbers': self.show_numbers,
            'header_html': self.header_html,
            'footer_html': self.footer_html,
        })

        fragment = Fragment(html)
        fragment.add_css_url(self.runtime.local_resource_url(self, self.css_path))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, self.js_path))
        fragment.initialize_js(
            'PBDashboardBlock', {
                'reportTemplate': report_template,
                'reportContentSelector': '.dashboard-report'
            })
        return fragment

    def validate_field_data(self, validation, data):
        """
        Validate this block's field data.
        """
        super(DashboardBlock, self).validate_field_data(validation, data)

        def add_error(msg):
            validation.add(ValidationMessage(ValidationMessage.ERROR, msg))

        try:
            list(self.get_mentoring_blocks(data.mentoring_ids, ignore_errors=False))
        except InvalidUrlName as e:
            add_error(_(u'Invalid block url_name given: "{bad_url_name}"').format(bad_url_name=six.text_type(e)))

        if data.exclude_questions:
            for key, value in six.iteritems(data.exclude_questions):
                if not isinstance(value, list):
                    add_error(
                        _(u"'Questions to be hidden' is malformed: value for key {key} is {value}, "
                          u"expected list of integers")
                        .format(key=key, value=value)
                    )

                if key not in data.mentoring_ids:
                    add_error(
                        _(u"'Questions to be hidden' is malformed: mentoring url_name {url_name} "
                          u"is not added to Dashboard")
                        .format(url_name=key)
                    )

        if data.average_labels:
            for key, value in six.iteritems(data.average_labels):
                if not isinstance(value, six.string_types):
                    add_error(
                        _(u"'Label for average value' is malformed: value for key {key} is {value}, expected string")
                        .format(key=key, value=value)
                    )

                if key not in data.mentoring_ids:
                    add_error(
                        _(u"'Label for average value' is malformed: mentoring url_name {url_name} "
                          u"is not added to Dashboard")
                        .format(url_name=key)
                    )

        if data.color_rules:
            try:
                self.parse_color_rules_str(data.color_rules, ignore_errors=False)
            except ValueError as e:
                add_error(six.text_type(e))

        if data.visual_rules:
            try:
                rules = json.loads(data.visual_rules)
            except ValueError as e:
                add_error(_(u"Visual rules contains an error: {error}").format(error=e))
            else:
                if not isinstance(rules, dict):
                    add_error(_(u"Visual rules should be a JSON dictionary/object: {...}"))
